package org.butor.spring;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.net.URL;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.FactoryBean;
import org.springframework.util.ResourceUtils;

public class StringResourceFactoryBean implements FactoryBean<String> {
	protected Logger logger = LoggerFactory.getLogger(getClass());

	private final String convertedResource;
	private String charSet = null;
	public StringResourceFactoryBean(String resourcePaths) {
		this(resourcePaths, null);
	}
	public StringResourceFactoryBean(String resourcePaths, String charSet) {
		this.charSet = charSet;
		String[] toks = resourcePaths.split(",");
		String res = null;
		for (String tok : toks) {
			try {
				logger.info("Getting resource from resourcePath={}, charSet={}...", tok, charSet);
				res = getResource(tok);
				break;
			} catch (IllegalArgumentException ex) {
				logger.warn("Failed to get resource from resourcePath={}! will try with next value if provided.", tok);
			}
		}
		if (res == null) {
			throw new IllegalArgumentException(String.format("Unable to read resource from : %s", resourcePaths));
		}
		convertedResource = res;
	}
	private String getResource(String resourcePath) {
		InputStreamReader is = null;
		try {
			URL u = ResourceUtils.getURL(resourcePath);
			if (charSet != null) {
				is = new InputStreamReader(u.openStream(), charSet);
			} else {
				is = new InputStreamReader(u.openStream());
			}
			StringWriter sw = new StringWriter();
			while(true) {
				int c = is.read();
				if (c == -1)
					break;
				sw.append((char)c);
			}
			return sw.toString();
		} catch (Exception e) {
			throw new IllegalArgumentException(String.format("Unable to read resource : %s",resourcePath),e);
		} finally {
			if (is != null) {
				try {
					is.close();
				} catch (IOException e) {
					throw new IllegalArgumentException(String.format("Unable to read resource : %s",resourcePath),e);
				}
			}
		}
		
	}
	
	@Override
	public String getObject() throws Exception {
		return convertedResource;
	}

	@Override
	public Class<?> getObjectType() {
		return String.class;
	}

	@Override
	public boolean isSingleton() {
		return true;
	}

}
